from typing import Annotated, Optional
from typer import Argument, Typer, Exit, launch
from rich.console import Console
from rich.syntax import Syntax
from rich.panel import Panel
from rich.padding import Padding
from pipconf.configs import PipConfigs
from pipconf.consts import PADDING, PADDING_LIST, ExitCodes, Chars, HelpPanels
from pipconf import __help__


app = Typer(rich_markup_mode='rich', help=__help__)
console = Console()
configs = PipConfigs()


@app.command(rich_help_panel=HelpPanels.DISPLAY)
def list():
    """Lists all available configs"""
    try:
        current = configs.current
    except EnvironmentError:
        current = None

    try:
        lines = []
        for path in configs.available_configs:
            if path != current:
                lines.append(
                    f'{Chars.EMPTY_CIRCLE.decode()} {path.name} ([grey42]{str(path)}[/])'
                )
            else:
                lines.append(
                    f'[green]{Chars.FILLED_CIRCLE.decode()} {path.name}[/] ([grey42]{str(path)}[/])'
                )

        console.print(
            Padding(
                f'Available configurations at [yellow]{configs.directory}[/]:',
                PADDING,
            )
        )
        console.print(
            Padding(
                '\n'.join(lines),
                PADDING_LIST,
            )
        )

    except EnvironmentError as exc:
        console.print(Padding(str(exc), PADDING), style='red')
        raise Exit(ExitCodes.NO_SUCH_FILE_OR_DIRECTORY)


@app.command(rich_help_panel=HelpPanels.DISPLAY)
def current():
    """Shows the currently active config file"""
    try:
        console.print(
            Padding(
                f'Current configuration is [yellow]{str(configs.current)}[/]!',
                PADDING,
            )
        )

    except EnvironmentError as exc:
        console.print(Padding(str(exc), PADDING), style='red')
        raise Exit(ExitCodes.NO_SUCH_FILE_OR_DIRECTORY)


@app.command(rich_help_panel=HelpPanels.DISPLAY)
def show(
    name: Annotated[Optional[str], Argument()] = None, local: bool = False
):
    """Shows a config file content"""
    try:
        if local:
            path = configs.local
        else:
            path = configs.get_path(name) if name else configs.current

        console.print(Panel(Syntax(configs.show(path), 'ini'), title=str(path)))

    except EnvironmentError as exc:
        console.print(Padding(str(exc), PADDING))
        raise Exit(ExitCodes.NO_SUCH_FILE_OR_DIRECTORY)


@app.command(rich_help_panel=HelpPanels.CHANGE)
def new(name: str, open: bool = False):
    """Creates a new config file"""
    try:
        path = configs.get_path(name)
        configs.create(path)
        console.print(
            Padding(f'Config file [green]{path.name}[/] created!', PADDING)
        )

        if open:
            exit_code = launch(str(path))
            if exit_code >= 0:
                launch(str(path), locate=True)

    except EnvironmentError as exc:
        console.print(Padding(str(exc), PADDING))
        raise Exit(ExitCodes.FILE_EXISTS)


@app.command(rich_help_panel=HelpPanels.CHANGE)
def set(name: str):
    """Select a configuration"""
    try:
        path = configs.get_path(name)
        configs.select(path)
        console.print(
            Padding(
                f'Configuration is now set to [yellow]{path.name}[/]!', PADDING
            )
        )

    except EnvironmentError as exc:
        console.print(Padding(str(exc), PADDING))
        raise Exit(ExitCodes.NO_SUCH_FILE_OR_DIRECTORY)


@app.command(rich_help_panel=HelpPanels.CHANGE)
def local():
    """Select a config file in current workdir"""
    try:
        local_config = configs.local
        configs.select(local_config)
        console.print(
            Padding(
                f'Configuration is now set to [yellow]{str(local_config)}[/]!',
                PADDING,
            )
        )

    except EnvironmentError as exc:
        console.print(Padding(str(exc), PADDING))
        raise Exit(ExitCodes.NO_SUCH_FILE_OR_DIRECTORY)



from pathlib import Path
from shutil import copyfile
from typing import List, Optional


class PipConfigs:
    def __init__(self) -> None:
        self._directory: Optional[Path] = None
        self._default_file: Optional[Path] = None
        self._extension = 'conf'
        self._config_file = 'pip.conf'
        self._template = Path(__file__).parent.joinpath(
            f'templates/{self._config_file}'
        )

    @property
    def directory(self) -> Path:
        if self._directory is None:
            self._directory = Path.home().joinpath('.pip')
            if not self._directory.exists():
                self._directory.mkdir(exist_ok=True)
        return self._directory

    @property
    def default_file(self) -> Path:
        if self._default_file is None:
            self._default_file = self.directory.joinpath(self._config_file)
        return self._default_file

    @property
    def current(self) -> Path:
        if not self.default_file.exists():
            raise EnvironmentError('No configuration found!')
        return self.default_file.readlink()

    @property
    def local(self) -> Path:
        cwd = Path.cwd()
        file_path = cwd.joinpath(self._config_file)
        if not file_path.exists():
            raise EnvironmentError(f'No configuration found at {str(cwd)}!')
        return file_path

    @property
    def available_configs(self) -> List[Path]:
        gotten_files = [
            path for path in self.directory.iterdir() if not path.is_symlink()
        ]
        if not gotten_files:
            raise EnvironmentError('No one configuration found!')
        return gotten_files

    def get_path(self, name: str) -> Path:
        if not name.endswith(self._extension):
            name = f'{name}.{self._extension}'
        return self.directory.joinpath(name)

    def select(self, path: Path):
        if not path.exists():
            raise EnvironmentError(f'The file {path} does not exist!')
        if all(
            [self.default_file.exists(), not self.default_file.is_symlink()]
        ):
            backup_path = self.default_file.parent.joinpath(
                f'pip.backup.{self._extension}'
            )
            copyfile(self.default_file, backup_path)
        self.default_file.unlink(missing_ok=True)
        self.default_file.symlink_to(path)

    def create(self, path: Path):
        if path.exists():
            raise EnvironmentError(f'The file {path} already exists!')
        copyfile(self._template, path)

    def show(self, path: Path) -> str:
        if not path.exists():
            raise EnvironmentError(f'The file {path} does not exist!')
        return path.read_text()



PADDING = (
    1,
    1,
)

PADDING_LIST = (
    1,
    1,
)


class Chars:
    FILLED_CIRCLE = b'\xe2\x97\x8f'
    EMPTY_CIRCLE = b'\xe2\x97\x8b'


class ExitCodes:
    NO_SUCH_FILE_OR_DIRECTORY = 2
    FILE_EXISTS = 17


class HelpPanels:
    DISPLAY = 'Display informations'
    CHANGE = 'Change configurations'
